﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace RGBConfigurator
{
    public partial class HostColorControl : UserControl
    {
        protected PerformanceCounter cpuCounter;



        public HostColorControl()
        {
            InitializeComponent();
            cpuCounter = new PerformanceCounter();

            cpuCounter.CategoryName = "Processor";
            cpuCounter.CounterName = "% Processor Time";
            cpuCounter.InstanceName = "_Total"; 
        }

        

        public bool cpuModeActive
        {
            get{
            return checkBoxUseCPULoadColor.Checked;
            }
        }

        public bool ambilightModeActive
        {
            get
            {
                return checkBoxAmbilight.Checked;
            }
        }

              

        public Color GetCPULoadColor()
        {
            int cpuLoad = (int)cpuCounter.NextValue();

            labelCPULoad.Text = "CPU load: " + cpuLoad + "%";

            Color c = Color.FromArgb((int)(cpuLoad*2.5), (int)(255-cpuLoad*2.5), 0);

            panelColor.BackColor = c;

            return c;
        }

        public Color GetAmbilightColor()
        {
            Bitmap img = CaptureScreen();
            int r = 0;
            int g = 0;
            int b = 0;
            int samples = 0;

            for (int x = 0; x < img.Width; x += 10)
            {
                for (int y = 0; y < img.Height; y += 10)
                {
                    Color pixelColor = img.GetPixel(x, y);
                    r += pixelColor.R;
                    g += pixelColor.G;
                    b += pixelColor.B;
                    samples++;
                }
            }
            Color c = Color.FromArgb(r / samples, g / samples, b / samples);
            
            panelColor.BackColor = c;

            return c;
        }

        private Bitmap CaptureScreen()
        {
            Rectangle screenSize = Screen.PrimaryScreen.Bounds;
            Bitmap target = new Bitmap(screenSize.Width, screenSize.Height/2);
            using (Graphics g = Graphics.FromImage(target))
            {
                g.CopyFromScreen(0, screenSize.Height / 2, 0, 0, new Size(screenSize.Width, screenSize.Height / 2));
            }
            return target;
        }


        private void checkBoxAmbilight_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxAmbilight.Checked)
            {
                checkBoxUseCPULoadColor.Checked = false;
            }
        }

        private void checkBoxUseCPULoadColor_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxUseCPULoadColor.Checked)
            {
                checkBoxAmbilight.Checked = false;
            }
        }
       
    }
}
