/*********************************************************************
 * This project is thus licensed with the German version of the
 * Creative Commons Attribution-Noncommercial-Share Alike 2.0 License.
 * 
 * What this means for you using this work is stated at: 
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.0/de/deed.en
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Steffen Schtte      01/28/2010    
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <stdio.h>
#include <EEP.h>

#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"

#include "1wire.h"


#pragma udata

/** D E F I N E S ************************************************************/



/** V A R I A B L E S ********************************************************/
//For USB stuff
byte syncCounter = 0;
DATA_PACKET dataPacket;


//For LED stuff
byte r;
byte g;
byte b;
byte rgbControl = 0; //Control variable for rgb color change

byte counter = 0;



//Mode stuff
byte modeButtonLast;
byte mode = 0;

//Host color control stuff
BOOL useHostColor = FALSE;
byte hostR = 0;
byte hostG = 0;
byte hostB = 0;
BOOL usbOnline = FALSE;
byte hostColorDelay = 0;



//Temerature sensor stuff
byte temp_byte;
float temp;
float temp_factor; //Faktor fr normalisierung
byte temp_n; //Temperatur normalisiert auf 0 bis 255 fr den definierten temperaturbereich
byte temp_n_target;


byte cfgData[14]; //Data array containing the color configuration


const rom byte temp_c[256][3] = 
{
{	0	,	0	,	255	},
{	0	,	4	,	255	},
{	0	,	8	,	255	},
{	0	,	12	,	255	},
{	0	,	16	,	255	},
{	0	,	20	,	255	},
{	0	,	24	,	255	},
{	0	,	28	,	255	},
{	0	,	32	,	255	},
{	0	,	36	,	255	},
{	0	,	40	,	255	},
{	0	,	44	,	255	},
{	0	,	48	,	255	},
{	0	,	52	,	255	},
{	0	,	56	,	255	},
{	0	,	60	,	255	},
{	0	,	64	,	255	},
{	0	,	68	,	255	},
{	0	,	72	,	255	},
{	0	,	76	,	255	},
{	0	,	80	,	255	},
{	0	,	84	,	255	},
{	0	,	88	,	255	},
{	0	,	92	,	255	},
{	0	,	96	,	255	},
{	0	,	100	,	255	},
{	0	,	104	,	255	},
{	0	,	108	,	255	},
{	0	,	112	,	255	},
{	0	,	116	,	255	},
{	0	,	120	,	255	},
{	0	,	124	,	255	},
{	0	,	128	,	255	},
{	0	,	132	,	255	},
{	0	,	136	,	255	},
{	0	,	140	,	255	},
{	0	,	144	,	255	},
{	0	,	148	,	255	},
{	0	,	152	,	255	},
{	0	,	156	,	255	},
{	0	,	160	,	255	},
{	0	,	164	,	255	},
{	0	,	168	,	255	},
{	0	,	172	,	255	},
{	0	,	176	,	255	},
{	0	,	180	,	255	},
{	0	,	184	,	255	},
{	0	,	188	,	255	},
{	0	,	192	,	255	},
{	0	,	196	,	255	},
{	0	,	200	,	255	},
{	0	,	204	,	255	},
{	0	,	208	,	255	},
{	0	,	212	,	255	},
{	0	,	216	,	255	},
{	0	,	220	,	255	},
{	0	,	224	,	255	},
{	0	,	228	,	255	},
{	0	,	232	,	255	},
{	0	,	236	,	255	},
{	0	,	240	,	255	},
{	0	,	244	,	255	},
{	0	,	248	,	255	},
{	0	,	252	,	255	},
{	0	,	255	,	255	},
{	0	,	255	,	251	},
{	0	,	255	,	247	},
{	0	,	255	,	243	},
{	0	,	255	,	239	},
{	0	,	255	,	235	},
{	0	,	255	,	231	},
{	0	,	255	,	227	},
{	0	,	255	,	223	},
{	0	,	255	,	219	},
{	0	,	255	,	215	},
{	0	,	255	,	211	},
{	0	,	255	,	207	},
{	0	,	255	,	203	},
{	0	,	255	,	199	},
{	0	,	255	,	195	},
{	0	,	255	,	191	},
{	0	,	255	,	187	},
{	0	,	255	,	183	},
{	0	,	255	,	179	},
{	0	,	255	,	175	},
{	0	,	255	,	171	},
{	0	,	255	,	167	},
{	0	,	255	,	163	},
{	0	,	255	,	159	},
{	0	,	255	,	155	},
{	0	,	255	,	151	},
{	0	,	255	,	147	},
{	0	,	255	,	143	},
{	0	,	255	,	139	},
{	0	,	255	,	135	},
{	0	,	255	,	131	},
{	0	,	255	,	127	},
{	0	,	255	,	123	},
{	0	,	255	,	119	},
{	0	,	255	,	115	},
{	0	,	255	,	111	},
{	0	,	255	,	107	},
{	0	,	255	,	103	},
{	0	,	255	,	99	},
{	0	,	255	,	95	},
{	0	,	255	,	91	},
{	0	,	255	,	87	},
{	0	,	255	,	83	},
{	0	,	255	,	79	},
{	0	,	255	,	75	},
{	0	,	255	,	71	},
{	0	,	255	,	67	},
{	0	,	255	,	63	},
{	0	,	255	,	59	},
{	0	,	255	,	55	},
{	0	,	255	,	51	},
{	0	,	255	,	47	},
{	0	,	255	,	43	},
{	0	,	255	,	39	},
{	0	,	255	,	35	},
{	0	,	255	,	31	},
{	0	,	255	,	27	},
{	0	,	255	,	23	},
{	0	,	255	,	19	},
{	0	,	255	,	15	},
{	0	,	255	,	11	},
{	0	,	255	,	7	},
{	0	,	255	,	3	},
{	0	,	255	,	0	},
{	4	,	255	,	0	},
{	8	,	255	,	0	},
{	12	,	255	,	0	},
{	16	,	255	,	0	},
{	20	,	255	,	0	},
{	24	,	255	,	0	},
{	28	,	255	,	0	},
{	32	,	255	,	0	},
{	36	,	255	,	0	},
{	40	,	255	,	0	},
{	44	,	255	,	0	},
{	48	,	255	,	0	},
{	52	,	255	,	0	},
{	56	,	255	,	0	},
{	60	,	255	,	0	},
{	64	,	255	,	0	},
{	68	,	255	,	0	},
{	72	,	255	,	0	},
{	76	,	255	,	0	},
{	80	,	255	,	0	},
{	84	,	255	,	0	},
{	88	,	255	,	0	},
{	92	,	255	,	0	},
{	96	,	255	,	0	},
{	100	,	255	,	0	},
{	104	,	255	,	0	},
{	108	,	255	,	0	},
{	112	,	255	,	0	},
{	116	,	255	,	0	},
{	120	,	255	,	0	},
{	124	,	255	,	0	},
{	128	,	255	,	0	},
{	132	,	255	,	0	},
{	136	,	255	,	0	},
{	140	,	255	,	0	},
{	144	,	255	,	0	},
{	148	,	255	,	0	},
{	152	,	255	,	0	},
{	156	,	255	,	0	},
{	160	,	255	,	0	},
{	164	,	255	,	0	},
{	168	,	255	,	0	},
{	172	,	255	,	0	},
{	176	,	255	,	0	},
{	180	,	255	,	0	},
{	184	,	255	,	0	},
{	188	,	255	,	0	},
{	192	,	255	,	0	},
{	196	,	255	,	0	},
{	200	,	255	,	0	},
{	204	,	255	,	0	},
{	208	,	255	,	0	},
{	212	,	255	,	0	},
{	216	,	255	,	0	},
{	220	,	255	,	0	},
{	224	,	255	,	0	},
{	228	,	255	,	0	},
{	232	,	255	,	0	},
{	236	,	255	,	0	},
{	240	,	255	,	0	},
{	244	,	255	,	0	},
{	248	,	255	,	0	},
{	252	,	255	,	0	},
{	255	,	255	,	0	},
{	255	,	251	,	0	},
{	255	,	247	,	0	},
{	255	,	243	,	0	},
{	255	,	239	,	0	},
{	255	,	235	,	0	},
{	255	,	231	,	0	},
{	255	,	227	,	0	},
{	255	,	223	,	0	},
{	255	,	219	,	0	},
{	255	,	215	,	0	},
{	255	,	211	,	0	},
{	255	,	207	,	0	},
{	255	,	203	,	0	},
{	255	,	199	,	0	},
{	255	,	195	,	0	},
{	255	,	191	,	0	},
{	255	,	187	,	0	},
{	255	,	183	,	0	},
{	255	,	179	,	0	},
{	255	,	175	,	0	},
{	255	,	171	,	0	},
{	255	,	167	,	0	},
{	255	,	163	,	0	},
{	255	,	159	,	0	},
{	255	,	155	,	0	},
{	255	,	151	,	0	},
{	255	,	147	,	0	},
{	255	,	143	,	0	},
{	255	,	139	,	0	},
{	255	,	135	,	0	},
{	255	,	131	,	0	},
{	255	,	127	,	0	},
{	255	,	123	,	0	},
{	255	,	119	,	0	},
{	255	,	115	,	0	},
{	255	,	111	,	0	},
{	255	,	107	,	0	},
{	255	,	103	,	0	},
{	255	,	99	,	0	},
{	255	,	95	,	0	},
{	255	,	91	,	0	},
{	255	,	87	,	0	},
{	255	,	83	,	0	},
{	255	,	79	,	0	},
{	255	,	75	,	0	},
{	255	,	71	,	0	},
{	255	,	67	,	0	},
{	255	,	63	,	0	},
{	255	,	59	,	0	},
{	255	,	55	,	0	},
{	255	,	51	,	0	},
{	255	,	47	,	0	},
{	255	,	43	,	0	},
{	255	,	39	,	0	},
{	255	,	35	,	0	},
{	255	,	31	,	0	},
{	255	,	27	,	0	},
{	255	,	23	,	0	},
{	255	,	22	,	0	},
{	255	,	21	,	0	},
{	255	,	20	,	0	},
{	255	,	19	,	0	},
{	255	,	18	,	0	}
};


/** P R I V A T E  P R O T O T Y P E S ***************************************/

void ServiceRequests(void);
void user_interrupt(void);

void rgb_init(void);
void fastRGB(void);
void slowRGB(void);
void nextRGB(void);
void constantColor(byte r, byte g, byte b);
void SaveConfig(void);
void LoadConfig(void);


/** D E F I N E S  **************************************************/
#pragma code


#define RED   PORTCbits.RC0
#define GREEN PORTCbits.RC1
#define BLUE  PORTCbits.RC2

#define MODE_BUTTON  !PORTBbits.RB4	

#define NUM_OF_MODES 7


/** D E C L A R A T I O N S **************************************************/
void UserInit(void)
{
 	byte init;
	
	ADCON1 = 0x0F;                 // Default all pins to digital
	
	TRISBbits.TRISB4 = 1;
	TRISBbits.TRISB2 = 0;
	TRISCbits.TRISC0 = 0;
	TRISCbits.TRISC1 = 0;
	TRISCbits.TRISC2 = 0;
	
   //Configure TIMER3
	T3CON = 0b00110001; //timer3 on, 1:2 prescaler = turnaround time ~45ms
	PIE2bits.TMR3IE = 1;
	INTCONbits.PEIE = 1;
	
	PIR2bits.TMR3IF = 1;
	
	//Configure TIMER0
	T0CON = 0b11000111;
	
	INTCONbits.GIE = 0;
	
	rgb_init();
	
	modeButtonLast = MODE_BUTTON;
	

	init = Read_b_eep(0);
	if(init != 42)
	{
		//Use default config
		cfgData[0] = 22; //temp min
		cfgData[1] = 38; //temp max
		cfgData[2] = 75; //RGB, constant color 1
		cfgData[3] = 0;
		cfgData[4] = 0;
		cfgData[5] = 0; //RGB, constant color 2
		cfgData[6] = 0;
		cfgData[7] = 255;
		cfgData[8] = 255; //RGB, constant color 3
		cfgData[9] = 255;
		cfgData[10] = 0;
		cfgData[11] = 255; //RGB, constant color 4
		cfgData[12] = 255;
		cfgData[13] = 255;
	}
	else
	{
		LoadConfig();	
	}


	
	//Read last selected mode from EEPROM
	mode = Read_b_eep(1);

	temp_factor = 255.0f / (cfgData[1] - cfgData[0]);


}//end UserInit

void SaveConfig(void)
{
	byte i;
	byte idx;

	//Set configuration byte
	Write_b_eep(0, 42); //Write 42 to indicate that a configuration has been made
    Busy_eep();
	for(i = 0; i<14;i++)
	{
		idx = i+2;
		Write_b_eep(idx, cfgData[i]);
	    Busy_eep();

	}
	
}

void LoadConfig(void)
{
	byte i;
	byte idx;

	for(i = 0; i<14;i++)
	{
		idx = i+2;
		cfgData[i] = Read_b_eep(idx);
	}
}


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
	byte count, pwm;

	RED   = TMR0L < r;GREEN = TMR0L < g;BLUE  = TMR0L < b;
   	   
	if(PIR2bits.TMR3IF)
    {
		user_interrupt();
		count++;
		if (count > 2 && mode == 2)
		{
			count = 0;
			
			OW_reset_pulse(r, g, b);  //That the color is passed is a workaround for PWM performance
		 	OW_write_byte (0xCC);
		 	OW_write_byte (0x44);
			OW_reset_pulse(r, g, b);
			OW_write_byte (0xCC);
			OW_write_byte (0xBE);
			temp_byte = OW_read_byte();
			RED   = TMR0L < r;GREEN = TMR0L < g;BLUE  = TMR0L < b;
			temp = temp_byte;
			temp = temp / 2;
			RED   = TMR0L < r;GREEN = TMR0L < g;BLUE  = TMR0L < b;
		}

		//Clear interrupt flag	
		PIR2bits.TMR3IF = 0;
   	}

}


//User interrupt service routine
void user_interrupt(void)
{
	byte i;
	if(usbOnline && useHostColor)
	{

			//Output host color
			if(hostColorDelay == 0)
			{
				r = hostR;
				g = hostG;
				b = hostB;
			}
			else
			{
				for(i=0;i<hostColorDelay;i++)
				{
					if(r < hostR)
						r++;
					else if(r > hostR)
						r--;
					if(g < hostG)
						g++;
					else if(g > hostG)
						g--;
					if(b < hostB)
						b++;
					else if(b > hostB)
						b--;
				}
			}
	}
	else
	{

		//Change mode if button stat has changed from 0 to 1
		if(MODE_BUTTON && !modeButtonLast)
		{	
			mode++;
			mode = mode % NUM_OF_MODES;
			
			switch(mode)
			{
				case 0:
					rgb_init();
					break;
				case 1:
					rgb_init();
					break;	
				case 2:
					temp_factor = 255.0f / (cfgData[1] - cfgData[0]);	
			}
			
			//Write selected mode to non-volatile EEPROM
			Write_b_eep(1, mode);
							
		}
		//Remember current button position
		modeButtonLast = MODE_BUTTON;
	

		switch(mode)
		{
			case 0:
				fastRGB();
				break;
			case 1:
				slowRGB();
				break;
			case 2:
				//Calculate red color target
				if(temp <= cfgData[0]) 
				{
					temp_n_target = 0;
				}
				else if (temp >= cfgData[1])
				{
					temp_n_target = 255;
				}
				else
				{
					temp_n_target = (temp - cfgData[0]) * temp_factor;
				}
	
				//Adjust temp_n to target one step
				if(temp_n_target > temp_n)
					temp_n++;
				else if(temp_n_target < temp_n)
					temp_n--;
	
	
				//Output the color
				constantColor(temp_c[temp_n][0], temp_c[temp_n][1], temp_c[temp_n][2]);
				break;
	
			case 3:
				constantColor(cfgData[2], cfgData[3],cfgData[4]);
				break;
			case 4:
				constantColor(cfgData[5], cfgData[6],cfgData[7]);
				break;
			case 5:
				constantColor(cfgData[8], cfgData[9],cfgData[10]);
				break;
			case 6:
				constantColor(cfgData[11], cfgData[12],cfgData[13]);
				break;		
		}
	}
}

void constantColor(byte red, byte green, byte blue)
{
	r = red;
	g = green;
	b = blue;	
}


//Initialize variables for starting an rgb change
void rgb_init()
{
	constantColor(255,0,0);
	rgbControl = 0;
}

void fastRGB()
{
	nextRGB();
}

void slowRGB()
{
	counter++;
	if(counter == 20)
	{
		counter = 0;
		nextRGB();	
	}	
}

void nextRGB()
{
		switch(rgbControl)
		{
			//Grn increment
			case 0:
				g++;
				if(g == 255)
				{
					rgbControl++;
				}
				break;	

			//Rot decrement
			case 1:
				r--;
				if(r == 0)
				{
					rgbControl++;
				}
				break;	

			//Blau increment
			case 2:
				b++;
				if(b == 255)
				{
					rgbControl++;
				}
				break;	

			//Grn decrement
			case 3:
				g--;
				if(g == 0)
				{
					rgbControl++;
				}
				break;	

			//Rot increment
			case 4:
				r++;
				if(r == 255)
				{
					rgbControl++;
				}
				break;	

			//Blau decrement
			case 5:
				b--;
				if(b == 0)
				{
					rgbControl = 0;
				}
				break;	
		

		}
	

}


/***** USB Routines ****/


void ProcessUSBIO(void)
{
	// User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) 
	{
		usbOnline = FALSE;
		return;
	}
	else
	{
		usbOnline = TRUE;
    	ServiceRequests();
	}
}//end ProcessIO




void ServiceRequests(void)
{
	byte i;
	byte length;
	
  	if(USBGenRead((byte*)&dataPacket,sizeof(dataPacket)))
    {
        syncCounter = 0;
        switch(dataPacket.CMD)
        {
            case STATE_UPDATE:
				//Get data from host
				useHostColor = dataPacket._byte[1];
				hostR = dataPacket._byte[2];
				hostG = dataPacket._byte[3];
				hostB = dataPacket._byte[4];
				hostColorDelay = dataPacket._byte[5];

				//Write data to host
                dataPacket._byte[1] = mode;
                dataPacket._byte[2] = temp_byte;
                dataPacket._byte[3] = temp_n_target;
                syncCounter=0x04;
                break;

			case GET_CONFIG:
				LoadConfig();
				for(i = 0; i<14;i++)
				{
                	dataPacket._byte[i+1] = cfgData[i];
				}
                syncCounter = 16;
				break;


			case SET_CONFIG:
				for(i = 0; i<14;i++)
				{
                	cfgData[i] = dataPacket._byte[i+1]; 
				}
				SaveConfig();
                syncCounter = 1;
				break;

            default:
                break;
        }//end switch()

        if(syncCounter != 0)
        {
            if(!mUSBGenTxIsBusy())
                USBGenWrite((byte*)&dataPacket,syncCounter);
        }//end if
    }//end if
    


}//end ServiceRequests


/** EOF user.c ***************************************************************/
